@@ -108,33 +108,19 @@ int import_write(const char *filename, sign_key *key, char *passphrase,
 static sign_key *dropbear_read(const char* filename) {
 
 	buffer * buf = NULL;
-	int len, maxlen;
-	FILE *fp;
 	sign_key *ret = NULL;
+	int type;
 
-	buf = buf_new(2000);
-	/* can't use buf_readfile since we might have "-" as filename */
-	if (strlen(filename) == 1 && filename[0] == '-') {
-		fp = stdin;
-	} else {
-		fp = fopen(filename, "r");
-	}
-	if (!fp) {
+	buf = buf_new(MAX_PRIVKEY_SIZE);
+	if (buf_readfile(buf, filename) == DROPBEAR_FAILURE) {
 		goto error;
 	}
 
-	do {
-		maxlen = buf->size - buf->pos;
-		len = fread(buf_getwriteptr(buf, maxlen), 1, maxlen, fp);
-		buf_incrwritepos(buf, len);
-	} while (len != maxlen && len > 0);
-
-	fclose(fp);
-
 	buf_setpos(buf, 0);
 	ret = new_sign_key();
 
-	if (buf_get_priv_key(buf, ret, DROPBEAR_SIGNKEY_ANY) == DROPBEAR_FAILURE){
+	type = DROPBEAR_SIGNKEY_ANY;
+	if (buf_get_priv_key(buf, ret, &type) == DROPBEAR_FAILURE){
 		goto error;
 	}
 	buf_free(buf);
@@ -171,14 +157,10 @@ static int dropbear_write(const char*filename, sign_key * key) {
 	}
 #endif
 
-	buf = buf_new(2000);
+	buf = buf_new(MAX_PRIVKEY_SIZE);
 	buf_put_priv_key(buf, key, keytype);
 
-	if (strlen(filename) == 1 && filename[0] == '-') {
-		fp = stdout;
-	} else {
-		fp = fopen(filename, "w");
-	}
+	fp = fopen(filename, "w");
 	if (!fp) {
 		ret = 0;
 		goto out;
@@ -529,6 +511,7 @@ static sign_key *openssh_read(const char *filename, char *passphrase)
 	char *errmsg;
 	char *modptr = NULL;
 	int modlen = -9999;
+	int type;
 
 	sign_key *retkey;
 	buffer * blobbuf = NULL;
@@ -675,7 +658,8 @@ static sign_key *openssh_read(const char *filename, char *passphrase)
 	 */
 	retkey = new_sign_key();
 	buf_setpos(blobbuf, 0);
-	if (buf_get_priv_key(blobbuf, retkey, DROPBEAR_SIGNKEY_ANY)
+	type = DROPBEAR_SIGNKEY_ANY;
+	if (buf_get_priv_key(blobbuf, retkey, &type)
 			!= DROPBEAR_SUCCESS) {
 		errmsg = "unable to create key structure";
 		sign_key_free(retkey);
@@ -715,10 +699,11 @@ static int openssh_write(const char *filename, sign_key *key,
 	unsigned char iv[8];
 	int ret = 0;
 	FILE *fp;
-	mp_int dmp1, dmq1, iqmp, tmpval; /* for rsa */
 	int keytype = -1;
 
 #ifdef DROPBEAR_RSA
+	mp_int dmp1, dmq1, iqmp, tmpval; /* for rsa */
+
 	if (key->rsakey != NULL) {
 		keytype = DROPBEAR_SIGNKEY_RSA;
 	}
