@@ -1,7 +1,20 @@
+/* LibTomCrypt, modular cryptographic library -- Tom St Denis
+ *
+ * LibTomCrypt is a library that provides various cryptographic
+ * algorithms in a highly modular and flexible manner.
+ *
+ * The library is free for all purposes without any express
+ * guarantee it works.
+ *
+ * Tom St Denis, tomstdenis@iahu.ca, http://libtomcrypt.org
+ */
 #include "mycrypt.h"
 
 #ifdef MDH
 
+/* max export size we'll encounter (smaller than this but lets round up a bit */
+#define DH_BUF_SIZE 1200
+
 /* This holds the key settings.  ***MUST*** be organized by size from smallest to largest. */
 static const struct {
     int size;
@@ -147,48 +160,48 @@ static int is_valid_idx(int n)
 int dh_test(void)
 {
     mp_int p, g, tmp;
-    int x, res, primality;
+    int x, err, primality;
 
-    if ((res = mp_init_multi(&p, &g, &tmp, NULL)) != MP_OKAY)                 { goto error; }
+    if ((err = mp_init_multi(&p, &g, &tmp, NULL)) != MP_OKAY)                 { goto error; }
 
     for (x = 0; sets[x].size != 0; x++) {
 #if 0
         printf("dh_test():testing size %d-bits\n", sets[x].size * 8);
 #endif
-        if ((res = mp_read_radix(&g,(char *)sets[x].base, 64)) != MP_OKAY)    { goto error; }
-        if ((res = mp_read_radix(&p,(char *)sets[x].prime, 64)) != MP_OKAY)   { goto error; }
+        if ((err = mp_read_radix(&g,(char *)sets[x].base, 64)) != MP_OKAY)    { goto error; }
+        if ((err = mp_read_radix(&p,(char *)sets[x].prime, 64)) != MP_OKAY)   { goto error; }
 
         /* ensure p is prime */
-        if ((res = is_prime(&p, &primality)) != CRYPT_OK)                     { goto done; }
+        if ((err = is_prime(&p, &primality)) != CRYPT_OK)                     { goto done; }
         if (primality == 0) {
-           res = CRYPT_FAIL_TESTVECTOR;
+           err = CRYPT_FAIL_TESTVECTOR;
            goto done;
         }
 
-        if ((res = mp_sub_d(&p, 1, &tmp)) != MP_OKAY)                         { goto error; }
-        if ((res = mp_div_2(&tmp, &tmp)) != MP_OKAY)                          { goto error; }
+        if ((err = mp_sub_d(&p, 1, &tmp)) != MP_OKAY)                         { goto error; }
+        if ((err = mp_div_2(&tmp, &tmp)) != MP_OKAY)                          { goto error; }
 
         /* ensure (p-1)/2 is prime */
-        if ((res = is_prime(&tmp, &primality)) != CRYPT_OK)                   { goto done; }
+        if ((err = is_prime(&tmp, &primality)) != CRYPT_OK)                   { goto done; }
         if (primality == 0) {
-           res = CRYPT_FAIL_TESTVECTOR;
+           err = CRYPT_FAIL_TESTVECTOR;
            goto done;
         }
 
         /* now see if g^((p-1)/2) mod p is in fact 1 */
-        if ((res = mp_exptmod(&g, &tmp, &p, &tmp)) != MP_OKAY)                { goto error; }
+        if ((err = mp_exptmod(&g, &tmp, &p, &tmp)) != MP_OKAY)                { goto error; }
         if (mp_cmp_d(&tmp, 1)) {
-           res = CRYPT_FAIL_TESTVECTOR;
+           err = CRYPT_FAIL_TESTVECTOR;
            goto done;
         }
     }
-    res = CRYPT_OK;
+    err = CRYPT_OK;
     goto done;
 error:
-    res = mpi_to_ltc_error(res);
+    err = mpi_to_ltc_error(err);
 done:
     mp_clear_multi(&tmp, &g, &p, NULL);
-    return res;
+    return err;
 }
 
 void dh_sizes(int *low, int *high)
@@ -216,10 +229,10 @@ int dh_get_size(dh_key *key)
 
 int dh_make_key(prng_state *prng, int wprng, int keysize, dh_key *key)
 {
-   unsigned char buf[512];
+   unsigned char *buf;
    unsigned long x;
    mp_int p, g;
-   int res, err;
+   int err;
 
    _ARGCHK(key  != NULL);
 
@@ -241,36 +254,47 @@ int dh_make_key(prng_state *prng, int wprng, int keysize, dh_key *key)
    }
    key->idx = x;
 
+   /* allocate buffer */
+   buf = XMALLOC(keysize);
+   if (buf == NULL) {
+      return CRYPT_MEM;
+   }
+
    /* make up random string */
    if (prng_descriptor[wprng].read(buf, keysize, prng) != (unsigned long)keysize) {
-      return CRYPT_ERROR_READPRNG;
+      err = CRYPT_ERROR_READPRNG; 
+      goto error2;
    }
 
    /* init parameters */
-   if ((res = mp_init_multi(&g, &p, &key->x, &key->y, NULL)) != MP_OKAY) {
-      return mpi_to_ltc_error(res);
+   if ((err = mp_init_multi(&g, &p, &key->x, &key->y, NULL)) != MP_OKAY) {
+      goto error;
    }
-   if ((res = mp_read_radix(&g, sets[key->idx].base, 64)) != MP_OKAY)      { goto error; }
-   if ((res = mp_read_radix(&p, sets[key->idx].prime, 64)) != MP_OKAY)     { goto error; }
+   if ((err = mp_read_radix(&g, sets[key->idx].base, 64)) != MP_OKAY)      { goto error; }
+   if ((err = mp_read_radix(&p, sets[key->idx].prime, 64)) != MP_OKAY)     { goto error; }
 
    /* load the x value */
-   if ((res = mp_read_unsigned_bin(&key->x, buf, keysize)) != MP_OKAY)     { goto error; }
-   if ((res = mp_exptmod(&g, &key->x, &p, &key->y)) != MP_OKAY)            { goto error; }
+   if ((err = mp_read_unsigned_bin(&key->x, buf, keysize)) != MP_OKAY)     { goto error; }
+   if ((err = mp_exptmod(&g, &key->x, &p, &key->y)) != MP_OKAY)            { goto error; }
    key->type = PK_PRIVATE;
 
-   if ((res = mp_shrink(&key->x)) != MP_OKAY)                              { goto error; }
-   if ((res = mp_shrink(&key->y)) != MP_OKAY)                              { goto error; }
+   if ((err = mp_shrink(&key->x)) != MP_OKAY)                              { goto error; }
+   if ((err = mp_shrink(&key->y)) != MP_OKAY)                              { goto error; }
 
    /* free up ram */
-   res = CRYPT_OK;
-   goto done2;
+   err = CRYPT_OK;
+   goto done;
 error:
-   res = mpi_to_ltc_error(res);
+   err = mpi_to_ltc_error(err);
+error2:
    mp_clear_multi(&key->x, &key->y, NULL);
-done2:
+done:
+#ifdef CLEAN_STACK
+   zeromem(buf, keysize);
+#endif
    mp_clear_multi(&p, &g, NULL);
-   zeromem(buf, sizeof(buf));
-   return res;
+   XFREE(buf);
+   return err;
 }
 
 void dh_free(dh_key *key)
@@ -279,55 +303,20 @@ void dh_free(dh_key *key)
    mp_clear_multi(&key->x, &key->y, NULL);
 }
 
-#define OUTPUT_BIGNUM(num, buf2, y, z)         \
-{                                              \
-      z = (unsigned long)mp_unsigned_bin_size(num);           \
-      STORE32L(z, buf2+y);                     \
-      y += 4;                                  \
-      if ((err = mp_to_unsigned_bin(num, buf2+y)) != MP_OKAY) { return mpi_to_ltc_error(err); }   \
-      y += z;                                  \
-}
-
-
-#define INPUT_BIGNUM(num, in, x, y)                              \
-{                                                                \
-     /* load value */                                            \
-     if (y + 4 > inlen) {                                        \
-        err = CRYPT_INVALID_PACKET;                            \
-        goto error;                                              \
-     }                                                           \
-     LOAD32L(x, in+y);                                           \
-     y += 4;                                                     \
-                                                                 \
-     /* sanity check... */                                       \
-     if (x+y > inlen) {                                          \
-        err = CRYPT_INVALID_PACKET;                            \
-        goto error;                                              \
-     }                                                           \
-                                                                 \
-     /* load it */                                               \
-     if ((err = mp_read_unsigned_bin(num, (unsigned char *)in+y, (int)x)) != MP_OKAY) {\
-        err = mpi_to_ltc_error(err);                                      \
-        goto error;                                              \
-     }                                                           \
-     y += x;                                                     \
-     if ((err = mp_shrink(num)) != MP_OKAY) {                            \
-        err = mpi_to_ltc_error(err);                                       \
-        goto error;                                              \
-     }                                                           \
-}
-
-
 int dh_export(unsigned char *out, unsigned long *outlen, int type, dh_key *key)
 {
-   unsigned char buf2[1536];
    unsigned long y, z;
    int err;
 
-   _ARGCHK(out != NULL);
+   _ARGCHK(out    != NULL);
    _ARGCHK(outlen != NULL);
-   _ARGCHK(key != NULL);
+   _ARGCHK(key    != NULL);
 
+   /* can we store the static header?  */
+   if (*outlen < (PACKET_SIZE + 2)) {
+      return CRYPT_BUFFER_OVERFLOW;
+   }
+   
    if (type == PK_PRIVATE && key->type != PK_PRIVATE) {
       return CRYPT_PK_NOT_PRIVATE;
    }
@@ -336,36 +325,22 @@ int dh_export(unsigned char *out, unsigned long *outlen, int type, dh_key *key)
    y = PACKET_SIZE;
 
    /* header */
-   buf2[y++] = type;
-   buf2[y++] = (unsigned char)(sets[key->idx].size / 8);
+   out[y++] = type;
+   out[y++] = (unsigned char)(sets[key->idx].size / 8);
 
    /* export y */
-   OUTPUT_BIGNUM(&key->y, buf2, y, z);
+   OUTPUT_BIGNUM(&key->y, out, y, z);
 
    if (type == PK_PRIVATE) {
       /* export x */
-      OUTPUT_BIGNUM(&key->x, buf2, y, z);
-   }
-
-   /* check for overflow */
-   if (*outlen < y) {
-      #ifdef CLEAN_STACK
-         zeromem(buf2, sizeof(buf2));
-      #endif
-      return CRYPT_BUFFER_OVERFLOW;
+      OUTPUT_BIGNUM(&key->x, out, y, z);
    }
 
    /* store header */
-   packet_store_header(buf2, PACKET_SECT_DH, PACKET_SUB_KEY);
+   packet_store_header(out, PACKET_SECT_DH, PACKET_SUB_KEY);
 
-   /* output it */
+   /* store len */
    *outlen = y;
-   memcpy(out, buf2, (size_t)y);
-
-   /* clear mem */
-#ifdef CLEAN_STACK
-   zeromem(buf2, sizeof(buf2));
-#endif
    return CRYPT_OK;
 }
 
@@ -374,11 +349,11 @@ int dh_import(const unsigned char *in, unsigned long inlen, dh_key *key)
    unsigned long x, y, s;
    int err;
 
-   _ARGCHK(in != NULL);
+   _ARGCHK(in  != NULL);
    _ARGCHK(key != NULL);
 
    /* make sure valid length */
-   if (2+PACKET_SIZE > inlen) {
+   if ((2+PACKET_SIZE) > inlen) {
       return CRYPT_INVALID_PACKET;
    }
 
@@ -421,10 +396,10 @@ int dh_import(const unsigned char *in, unsigned long inlen, dh_key *key)
    }
 
    /* load public value g^x mod p*/
-   INPUT_BIGNUM(&key->y, in, x, y);
+   INPUT_BIGNUM(&key->y, in, x, y, inlen);
 
    if (key->type == PK_PRIVATE) {
-      INPUT_BIGNUM(&key->x, in, x, y);
+      INPUT_BIGNUM(&key->x, in, x, y, inlen);
    }
 
    /* eliminate private key if public */
@@ -443,12 +418,12 @@ int dh_shared_secret(dh_key *private_key, dh_key *public_key,
 {
    mp_int tmp, p;
    unsigned long x;
-   int res;
+   int err;
 
    _ARGCHK(private_key != NULL);
    _ARGCHK(public_key  != NULL);
-   _ARGCHK(out != NULL);
-   _ARGCHK(outlen != NULL);
+   _ARGCHK(out         != NULL);
+   _ARGCHK(outlen      != NULL);
 
    /* types valid? */
    if (private_key->type != PK_PRIVATE) {
@@ -461,28 +436,28 @@ int dh_shared_secret(dh_key *private_key, dh_key *public_key,
    }
 
    /* compute y^x mod p */
-   if (mp_init_multi(&tmp, &p, NULL) != MP_OKAY) {
-      return CRYPT_MEM;
+   if ((err = mp_init_multi(&tmp, &p, NULL)) != MP_OKAY) {
+      return mpi_to_ltc_error(err);
    }
 
-   if (mp_read_radix(&p, (char *)sets[private_key->idx].prime, 64) != MP_OKAY)     { goto error; }
-   if (mp_exptmod(&public_key->y, &private_key->x, &p, &tmp) != MP_OKAY)           { goto error; }
+   if ((err = mp_read_radix(&p, (char *)sets[private_key->idx].prime, 64)) != MP_OKAY)     { goto error; }
+   if ((err = mp_exptmod(&public_key->y, &private_key->x, &p, &tmp)) != MP_OKAY)           { goto error; }
 
    /* enough space for output? */
    x = (unsigned long)mp_unsigned_bin_size(&tmp);
    if (*outlen < x) {
-      res = CRYPT_BUFFER_OVERFLOW;
+      err = CRYPT_BUFFER_OVERFLOW;
       goto done;
    }
-   if (mp_to_unsigned_bin(&tmp, out) != MP_OKAY)                                   { goto error; }
+   if ((err = mp_to_unsigned_bin(&tmp, out)) != MP_OKAY)                                   { goto error; }
    *outlen = x;
-   res = CRYPT_OK;
+   err = CRYPT_OK;
    goto done;
 error:
-   res = CRYPT_MEM;
+   err = mpi_to_ltc_error(err);
 done:
    mp_clear_multi(&p, &tmp, NULL);
-   return res;
+   return err;
 }
 
 #include "dh_sys.c"
