@@ -1,3 +1,13 @@
+/* LibTomCrypt, modular cryptographic library -- Tom St Denis
+ *
+ * LibTomCrypt is a library that provides various cryptographic
+ * algorithms in a highly modular and flexible manner.
+ *
+ * The library is free for all purposes without any express
+ * guarantee it works.
+ *
+ * Tom St Denis, tomstdenis@iahu.ca, http://libtomcrypt.org
+ */
 /* MD2 (RFC 1319) hash function implementation by Tom St Denis */
 #include "mycrypt.h"
 
@@ -9,6 +19,13 @@ const struct _hash_descriptor md2_desc =
     7,
     16,
     16,
+
+    /* DER encoding */
+    { 0x30, 0x20, 0x30, 0x0C, 0x06, 0x08, 0x2A, 0x86, 
+      0x48, 0x86, 0xF7, 0x0D, 0x02, 0x02, 0x05, 0x00, 
+      0x04, 0x10 },
+    18,
+
     &md2_init,
     &md2_process,
     &md2_done,
@@ -55,7 +72,7 @@ static void md2_compress(hash_state *md)
 {
    int j, k;
    unsigned char t;
-
+   
    /* copy block */
    for (j = 0; j < 16; j++) {
        md->md2.X[16+j] = md->md2.buf[j];
@@ -73,7 +90,7 @@ static void md2_compress(hash_state *md)
    }
 }
 
-void md2_init(hash_state *md)
+int md2_init(hash_state *md)
 {
    _ARGCHK(md != NULL);
 
@@ -82,6 +99,7 @@ void md2_init(hash_state *md)
    zeromem(md->md2.chksum, sizeof(md->md2.chksum));
    zeromem(md->md2.buf, sizeof(md->md2.buf));
    md->md2.curlen = 0;
+   return CRYPT_OK;
 }
 
 int md2_process(hash_state *md, const unsigned char *buf, unsigned long len)
@@ -94,7 +112,7 @@ int md2_process(hash_state *md, const unsigned char *buf, unsigned long len)
     }                                                                                       
     while (len > 0) {
         n = MIN(len, (16 - md->md2.curlen));
-        memcpy(md->md2.buf + md->md2.curlen, buf, (size_t)n);
+        XMEMCPY(md->md2.buf + md->md2.curlen, buf, (size_t)n);
         md->md2.curlen += n;
         buf            += n;
         len            -= n;
@@ -132,11 +150,11 @@ int md2_done(hash_state * md, unsigned char *hash)
     md2_update_chksum(md);
 
     /* hash checksum */
-    memcpy(md->md2.buf, md->md2.chksum, 16);
+    XMEMCPY(md->md2.buf, md->md2.chksum, 16);
     md2_compress(md);
 
     /* output is lower 16 bytes of X */
-    memcpy(hash, md->md2.X, 16);
+    XMEMCPY(hash, md->md2.X, 16);
 
 #ifdef CLEAN_STACK
     zeromem(md, sizeof(hash_state));
