@@ -16,13 +16,16 @@ extern "C" {
 #endif
 
 /* version */
-#define CRYPT   0x0083
-#define SCRYPT  "0.83"
+#define CRYPT   0x0099
+#define SCRYPT  "0.99"
 
 /* max size of either a cipher/hash block or symmetric key [largest of the two] */
-#define MAXBLOCKSIZE           128
+#define MAXBLOCKSIZE  64
+
+/* descriptor table size */
+/* Dropbear change - this should be smaller, saves some size */
+#define TAB_SIZE    4
 
-/* ch1-01-1 */
 /* error codes [will be expanded in future releases] */
 enum {
    CRYPT_OK=0,             /* Result OK */
@@ -49,6 +52,7 @@ enum {
    CRYPT_PK_NOT_PRIVATE,   /* Requires a private PK key */
 
    CRYPT_INVALID_ARG,      /* Generic invalid argument */
+   CRYPT_FILE_NOTFOUND,    /* File Not Found */
 
    CRYPT_PK_INVALID_TYPE,  /* Invalid type of PK key */
    CRYPT_PK_INVALID_SYSTEM,/* Invalid PK system specified */
@@ -58,7 +62,6 @@ enum {
 
    CRYPT_INVALID_PRIME_SIZE/* Invalid size of prime requested */
 };
-/* ch1-01-1 */
 
 #include <mycrypt_cfg.h>
 #include <mycrypt_macros.h>
@@ -66,10 +69,9 @@ enum {
 #include <mycrypt_hash.h>
 #include <mycrypt_prng.h>
 #include <mycrypt_pk.h>
-#include <mycrypt_gf.h>
 #include <mycrypt_misc.h>
-#include <mycrypt_kr.h>
 #include <mycrypt_argchk.h>
+#include <mycrypt_pkcs.h>
 
 #ifdef __cplusplus
    }
