@@ -8,19 +8,33 @@
 /* this will sort out which stuff based on the user-config in options.h */
 #include "../options.h"
 
-#ifdef CRYPT
-	#error mycrypt_custom.h should be included before mycrypt.h
+/* macros for various libc functions you can change for embedded targets */
+#define XMALLOC  malloc
+#define XREALLOC realloc
+#define XCALLOC  calloc
+#define XFREE    free
+
+#define XMEMSET  memset
+#define XMEMCPY  memcpy
+
+#define XCLOCK   clock
+#define XCLOCKS_PER_SEC CLOCKS_PER_SEC
+
+#ifdef DROPBEAR_SMALL_CODE
+#define SMALL_CODE
 #endif
 
-#define XMALLOC malloc
-#define XREALLOC realloc
-#define XCALLOC calloc
-#define XFREE free
-#define XCLOCK clock
-#define XCLOCKS_PER_SEC CLOCKS_PER_SEC
-#define SMALL_CODE
+/* Enable self-test test vector checking */
+/* Not for dropbear */
+//#define LTC_TEST
 
-/* #define LTC_TEST */
+/* clean the stack of functions which put private information on stack */
+// #define CLEAN_STACK
+
+/* disable all file related functions */
+// #define NO_FILE
+
+#define CLEAN_STACK
 
 #ifdef DROPBEAR_BLOWFISH_CBC
 #define BLOWFISH
@@ -32,6 +46,7 @@
 
 #ifdef DROPBEAR_TWOFISH128_CBC
 #define TWOFISH
+
 /* enabling just TWOFISH_SMALL will make the binary ~1kB smaller, turning on
  * TWOFISH_TABLES will make it a few kB bigger, but perhaps reduces runtime
  * memory usage? */
@@ -42,7 +57,6 @@
 #ifdef DROPBEAR_3DES_CBC
 #define DES
 #endif
-
 #define CBC
 
 #if defined(DROPBEAR_DSS) && defined(DSS_PROTOK)
@@ -56,10 +70,11 @@
 #endif
 
 #define HMAC
-#define OMAC
+
+/* Various tidbits of modern neatoness */
 #define BASE64
 
-#include <mycrypt.h>
+#define FORTUNA_POOLS 0
 
 #endif
 
