@@ -1,3 +1,15 @@
+/* LibTomCrypt, modular cryptographic library -- Tom St Denis
+ *
+ * LibTomCrypt is a library that provides various cryptographic
+ * algorithms in a highly modular and flexible manner.
+ *
+ * The library is free for all purposes without any express
+ * guarantee it works.
+ *
+ * Tom St Denis, tomstdenis@iahu.ca, http://libtomcrypt.org
+ */
+
+/* RC6 code by Tom St Denis */
 #include "mycrypt.h"
 
 #ifdef RC6
@@ -59,7 +71,7 @@ int rc6_setup(const unsigned char *key, int keylen, int num_rounds, symmetric_ke
     }
 
     /* setup the S array */
-    memcpy(S, stab, 44 * sizeof(stab[0]));
+    XMEMCPY(S, stab, 44 * sizeof(stab[0]));
 
     /* mix buffer */
     s = 3 * MAX(44, j);
@@ -240,6 +252,23 @@ int rc6_test(void)
 
       /* compare */
       if (memcmp(tmp[0], tests[x].ct, 16) || memcmp(tmp[1], tests[x].pt, 16)) {
+#if 0
+         printf("\n\nFailed test %d\n", x);
+         if (memcmp(tmp[0], tests[x].ct, 16)) {
+            printf("Ciphertext:  ");
+            for (y = 0; y < 16; y++) printf("%02x ", tmp[0][y]);
+            printf("\nExpected  :  ");
+            for (y = 0; y < 16; y++) printf("%02x ", tests[x].ct[y]);
+            printf("\n");
+         }
+         if (memcmp(tmp[1], tests[x].pt, 16)) {
+            printf("Plaintext:  ");
+            for (y = 0; y < 16; y++) printf("%02x ", tmp[0][y]);
+            printf("\nExpected :  ");
+            for (y = 0; y < 16; y++) printf("%02x ", tests[x].pt[y]);
+            printf("\n");
+         }
+#endif
          return CRYPT_FAIL_TESTVECTOR;
       }
 
