@@ -1,9 +1,11 @@
+#include <tommath.h>
+#ifdef BN_MP_LSHD_C
 /* LibTomMath, multiple-precision integer library -- Tom St Denis
  *
- * LibTomMath is library that provides for multiple-precision
+ * LibTomMath is a library that provides multiple-precision
  * integer arithmetic as well as number theoretic functionality.
  *
- * The library is designed directly after the MPI library by
+ * The library was designed directly after the MPI library by
  * Michael Fromberger but has been written from scratch with
  * additional optimizations in place.
  *
@@ -12,11 +14,9 @@
  *
  * Tom St Denis, tomstdenis@iahu.ca, http://math.libtomcrypt.org
  */
-#include <tommath.h>
 
 /* shift left a certain amount of digits */
-int
-mp_lshd (mp_int * a, int b)
+int mp_lshd (mp_int * a, int b)
 {
   int     x, res;
 
@@ -60,3 +60,4 @@ mp_lshd (mp_int * a, int b)
   }
   return MP_OKAY;
 }
+#endif
