@@ -1,3 +1,5 @@
+#include <tommath.h>
+#ifdef BN_MP_PRIME_FERMAT_C
 /* LibTomMath, multiple-precision integer library -- Tom St Denis
  *
  * LibTomMath is a library that provides multiple-precision
@@ -12,7 +14,6 @@
  *
  * Tom St Denis, tomstdenis@iahu.ca, http://math.libtomcrypt.org
  */
-#include <tommath.h>
 
 /* performs one Fermat test.
  * 
@@ -22,14 +23,13 @@
  *
  * Sets result to 1 if the congruence holds, or zero otherwise.
  */
-int
-mp_prime_fermat (mp_int * a, mp_int * b, int *result)
+int mp_prime_fermat (mp_int * a, mp_int * b, int *result)
 {
   mp_int  t;
   int     err;
 
   /* default to composite  */
-  *result = 0;
+  *result = MP_NO;
 
   /* ensure b > 1 */
   if (mp_cmp_d(b, 1) != MP_GT) {
@@ -48,10 +48,11 @@ mp_prime_fermat (mp_int * a, mp_int * b, int *result)
 
   /* is it equal to b? */
   if (mp_cmp (&t, b) == MP_EQ) {
-    *result = 1;
+    *result = MP_YES;
   }
 
   err = MP_OKAY;
 __T:mp_clear (&t);
   return err;
 }
+#endif
