@@ -1,3 +1,5 @@
+#include <tommath.h>
+#ifdef BN_MP_REDUCE_2K_SETUP_C
 /* LibTomMath, multiple-precision integer library -- Tom St Denis
  *
  * LibTomMath is a library that provides multiple-precision
@@ -12,7 +14,6 @@
  *
  * Tom St Denis, tomstdenis@iahu.ca, http://math.libtomcrypt.org
  */
-#include <tommath.h>
 
 /* determines the setup value */
 int 
@@ -40,3 +41,4 @@ mp_reduce_2k_setup(mp_int *a, mp_digit *d)
    mp_clear(&tmp);
    return MP_OKAY;
 }
+#endif
