@@ -1,3 +1,5 @@
+#include <tommath.h>
+#ifdef BN_MP_ZERO_C
 /* LibTomMath, multiple-precision integer library -- Tom St Denis
  *
  * LibTomMath is a library that provides multiple-precision
@@ -12,7 +14,6 @@
  *
  * Tom St Denis, tomstdenis@iahu.ca, http://math.libtomcrypt.org
  */
-#include <tommath.h>
 
 /* set to zero */
 void
@@ -22,3 +23,4 @@ mp_zero (mp_int * a)
   a->used = 0;
   memset (a->dp, 0, sizeof (mp_digit) * a->alloc);
 }
+#endif
