@@ -36,8 +36,6 @@
 #include "channel.h"
 
 static void read_packet_init();
-static void process_postauth_packet(unsigned int type);
-static void recv_unimplemented();
 static void writemac(buffer * outputbuffer, buffer * clearwritebuf);
 static int checkmac(buffer* hashbuf, buffer* readbuf);
 
@@ -52,9 +50,9 @@ static void buf_compress(buffer * dest, buffer * src, unsigned int len);
 void write_packet() {
 
 	int len, written;
-	buffer * writebuf;
+	buffer * writebuf = NULL;
 	
-	TRACE(("enter write_packet"));
+	TRACE(("enter write_packet"))
 	assert(!isempty(&ses.writequeue));
 
 	/* Get the next buffer in the queue of encrypted packets to write*/
@@ -67,7 +65,7 @@ void write_packet() {
 
 	if (written < 0) {
 		if (errno == EINTR) {
-			TRACE(("leave writepacket: EINTR"));
+			TRACE(("leave writepacket: EINTR"))
 			return;
 		} else {
 			dropbear_exit("error writing");
@@ -75,19 +73,20 @@ void write_packet() {
 	} 
 
 	if (written == 0) {
-		session_remoteclosed();
+		ses.remoteclosed();
 	}
 
 	if (written == len) {
 		/* We've finished with the packet, free it */
 		dequeue(&ses.writequeue);
 		buf_free(writebuf);
+		writebuf = NULL;
 	} else {
 		/* More packet left to write, leave it in the queue for later */
 		buf_incrpos(writebuf, written);
 	}
 
-	TRACE(("leave write_packet"));
+	TRACE(("leave write_packet"))
 }
 
 /* Non-blocking function reading available portion of a packet into the
@@ -99,7 +98,7 @@ void read_packet() {
 	unsigned int maxlen;
 	unsigned char blocksize;
 
-	TRACE(("enter read_packet"));
+	TRACE(("enter read_packet"))
 	blocksize = ses.keys->recv_algo_crypt->blocksize;
 	
 	if (ses.readbuf == NULL || ses.readbuf->len < blocksize) {
@@ -112,7 +111,7 @@ void read_packet() {
 		/* If we don't have the length of decryptreadbuf, we didn't read
 		 * a whole blocksize and should exit */
 		if (ses.decryptreadbuf->len == 0) {
-			TRACE(("leave read_packet: packetinit done"));
+			TRACE(("leave read_packet: packetinit done"))
 			return;
 		}
 	}
@@ -124,12 +123,12 @@ void read_packet() {
 	len = read(ses.sock, buf_getptr(ses.readbuf, maxlen), maxlen);
 
 	if (len == 0) {
-		session_remoteclosed();
+		ses.remoteclosed();
 	}
 
 	if (len < 0) {
 		if (errno == EINTR || errno == EAGAIN) {
-			TRACE(("leave read_packet: EINTR or EAGAIN"));
+			TRACE(("leave read_packet: EINTR or EAGAIN"))
 			return;
 		} else {
 			dropbear_exit("error reading: %s", strerror(errno));
@@ -141,10 +140,10 @@ void read_packet() {
 	if ((unsigned int)len == maxlen) {
 		/* The whole packet has been read */
 		decrypt_packet();
-		/* The main select() loop in session.h will process_packet() to
+		/* The main select() loop process_packet() to
 		 * handle the packet contents... */
 	}
-	TRACE(("leave read_packet"));
+	TRACE(("leave read_packet"))
 }
 
 /* Function used to read the initial portion of a packet, and determine the
@@ -173,11 +172,11 @@ static void read_packet_init() {
 	len = read(ses.sock, buf_getwriteptr(ses.readbuf, maxlen),
 			maxlen);
 	if (len == 0) {
-		session_remoteclosed();
+		ses.remoteclosed();
 	}
 	if (len < 0) {
 		if (errno == EINTR) {
-			TRACE(("leave read_packet_init: EINTR"));
+			TRACE(("leave read_packet_init: EINTR"))
 			return;
 		}
 		dropbear_exit("error reading: %s", strerror(errno));
@@ -231,7 +230,7 @@ void decrypt_packet() {
 	unsigned int padlen;
 	unsigned int len;
 
-	TRACE(("enter decrypt_packet"));
+	TRACE(("enter decrypt_packet"))
 	blocksize = ses.keys->recv_algo_crypt->blocksize;
 	macsize = ses.keys->recv_algo_mac->hashsize;
 
@@ -306,7 +305,7 @@ void decrypt_packet() {
 
 	ses.recvseq++;
 
-	TRACE(("leave decrypt_packet"));
+	TRACE(("leave decrypt_packet"))
 }
 
 /* Checks the mac in hashbuf, for the data in readbuf.
@@ -403,182 +402,8 @@ static buffer* buf_decompress(buffer* buf, unsigned int len) {
 #endif
 
 
-/* process a decrypted packet, call the appropriate handler */
-void process_packet() {
 
-	unsigned char type;
-
-	TRACE(("enter process_packet"));
-
-	type = buf_getbyte(ses.payload);
-	TRACE(("process_packet: packet type = %d", type));
-
-	/* these packets we can receive at any time, regardless of expecting
-	 * other packets: */
-	switch(type) {
-
-		case SSH_MSG_IGNORE:
-		case SSH_MSG_DEBUG:
-			TRACE(("received SSH_MSG_IGNORE or SSH_MSG_DEBUG"));
-			goto out;
-
-		case SSH_MSG_UNIMPLEMENTED:
-			/* debugging XXX */
-			TRACE(("SSH_MSG_UNIMPLEMENTED"));
-			dropbear_exit("received SSH_MSG_UNIMPLEMENTED");
-			
-		case SSH_MSG_DISCONNECT:
-			/* TODO cleanup? */
-			dropbear_close("Disconnect received");
-	}
-
-	/* Check if we should ignore this packet. Used currently only for
-	 * KEX code, with first_kex_packet_follows */
-	if (ses.ignorenext) {
-		TRACE(("Ignoring packet, type = %d", type));
-		ses.ignorenext = 0;
-		goto out;
-	}
-
-	/* check that we aren't expecting a particular packet */
-	if (ses.expecting && ses.expecting != type) {
-		/* TODO send disconnect? */
-		dropbear_exit("unexpected packet type %d, expected %d", type,
-				ses.expecting);
-	}
-
-	/* handle the packet depending on type */
-	ses.expecting = 0;
-
-	switch (type) {
-
-		case SSH_MSG_SERVICE_REQUEST:
-			recv_msg_service_request();
-			break;
-
-		case SSH_MSG_USERAUTH_REQUEST:
-			recv_msg_userauth_request();
-			break;
-			
-		case SSH_MSG_KEXINIT:
-			recv_msg_kexinit();
-			break;
-
-		case SSH_MSG_KEXDH_INIT:
-			recv_msg_kexdh_init();
-			break;
-
-		case SSH_MSG_NEWKEYS:
-			recv_msg_newkeys();
-			break;
-
-		/* this is ugly, need to make a cleaner way to do it */
-		case SSH_MSG_CHANNEL_DATA:
-		case SSH_MSG_CHANNEL_WINDOW_ADJUST:
-		case SSH_MSG_CHANNEL_REQUEST:
-		case SSH_MSG_CHANNEL_OPEN:
-		case SSH_MSG_CHANNEL_EOF:
-		case SSH_MSG_CHANNEL_CLOSE:
-		case SSH_MSG_CHANNEL_OPEN_CONFIRMATION:
-		case SSH_MSG_CHANNEL_OPEN_FAILURE:
-		case SSH_MSG_GLOBAL_REQUEST:
-			/* these should be checked for authdone below */
-			process_postauth_packet(type);
-			break;
 	
-		default:
-			/* TODO this possibly should be handled */
-			TRACE(("preauth unknown packet"));
-			recv_unimplemented();
-			break;
-	}
-
-out:
-	buf_free(ses.payload);
-	ses.payload = NULL;
-
-	TRACE(("leave process_packet"));
-}
-
-/* process a packet, and also check that auth has been done */
-static void process_postauth_packet(unsigned int type) {
-
-	/* messages following here require userauth before use */
-
-	/* IF YOU ADD MORE PACKET TYPES, MAKE SURE THEY'RE ALSO ADDED TO THE LIST
-	 * IN process_packet() XXX XXX XXX */
-	if (!ses.authstate.authdone) {
-		dropbear_exit("received message %d before userauth", type);
-	}
-
-	switch (type) {
-
-		case SSH_MSG_CHANNEL_DATA:
-			recv_msg_channel_data();
-			break;
-
-		case SSH_MSG_CHANNEL_WINDOW_ADJUST:
-			recv_msg_channel_window_adjust();
-			break;
-
-#ifndef DISABLE_REMOTETCPFWD
-		case SSH_MSG_GLOBAL_REQUEST:
-			/* currently only used for remote tcp, so we cheat a little */
-			recv_msg_global_request_remotetcp();
-			break;
-#endif
-
-		case SSH_MSG_CHANNEL_REQUEST:
-			recv_msg_channel_request();
-			break;
-
-		case SSH_MSG_CHANNEL_OPEN:
-			recv_msg_channel_open();
-			break;
-
-		case SSH_MSG_CHANNEL_EOF:
-			recv_msg_channel_eof();
-			break;
-
-		case SSH_MSG_CHANNEL_CLOSE:
-			recv_msg_channel_close();
-			break;
-
-#ifdef USE_LISTENERS /* for x11, tcp fwd etc */
-		case SSH_MSG_CHANNEL_OPEN_CONFIRMATION:
-			recv_msg_channel_open_confirmation();
-			break;
-			
-		case SSH_MSG_CHANNEL_OPEN_FAILURE:
-			recv_msg_channel_open_failure();
-			break;
-#endif
-			
-		default:
-			TRACE(("unknown packet()"));
-			recv_unimplemented();
-			break;
-	}
-}
-
-/* This must be called directly after receiving the unimplemented packet.
- * Isn't the most clean implementation, it relies on packet processing
- * occurring directly after decryption. This is reasonably valid, since
- * there is only a single decryption buffer */
-static void recv_unimplemented() {
-
-	CHECKCLEARTOWRITE();
-
-	buf_putbyte(ses.writepayload, SSH_MSG_UNIMPLEMENTED);
-	/* the decryption routine increments the sequence number, we must
-	 * decrement */
-	buf_putint(ses.writepayload, ses.recvseq - 1);
-
-	encrypt_packet();
-}
-	
-
-
 /* encrypt the writepayload, putting into writebuf, ready for write_packet()
  * to put on the wire */
 void encrypt_packet() {
@@ -588,8 +413,8 @@ void encrypt_packet() {
 	buffer * writebuf; /* the packet which will go on the wire */
 	buffer * clearwritebuf; /* unencrypted, possibly compressed */
 	
-	TRACE(("enter encrypt_packet()"));
-	TRACE(("encrypt_packet type is %d", ses.writepayload->data[0]));
+	TRACE(("enter encrypt_packet()"))
+	TRACE(("encrypt_packet type is %d", ses.writepayload->data[0]))
 	blocksize = ses.keys->trans_algo_crypt->blocksize;
 	macsize = ses.keys->trans_algo_mac->hashsize;
 
@@ -620,6 +445,10 @@ void encrypt_packet() {
 	}
 
 	/* finished with payload */
+	buf_burn(ses.writepayload); /* XXX This is probably a good idea, and isn't
+								   _that_ likely to hurt performance too badly.
+								   Buffers can have cleartext passwords etc, or
+								   other sensitive data */
 	buf_setpos(ses.writepayload, 0);
 	buf_setlen(ses.writepayload, 0);
 
@@ -675,6 +504,7 @@ void encrypt_packet() {
 
 	/* clearwritebuf is finished with */
 	buf_free(clearwritebuf);
+	clearwritebuf = NULL;
 
 	/* enqueue the packet for sending */
 	buf_setpos(writebuf, 0);
@@ -684,7 +514,7 @@ void encrypt_packet() {
 	ses.kexstate.datatrans += writebuf->len;
 	ses.transseq++;
 
-	TRACE(("leave encrypt_packet()"));
+	TRACE(("leave encrypt_packet()"))
 }
 
 
@@ -696,7 +526,7 @@ static void writemac(buffer * outputbuffer, buffer * clearwritebuf) {
 	unsigned long hashsize;
 	hmac_state hmac;
 
-	TRACE(("enter writemac"));
+	TRACE(("enter writemac"))
 
 	macsize = ses.keys->trans_algo_mac->hashsize;
 
@@ -731,7 +561,7 @@ static void writemac(buffer * outputbuffer, buffer * clearwritebuf) {
 		}
 		buf_incrwritepos(outputbuffer, macsize);
 	}
-	TRACE(("leave writemac"));
+	TRACE(("leave writemac"))
 }
 
 #ifndef DISABLE_ZLIB
@@ -742,7 +572,7 @@ static void buf_compress(buffer * dest, buffer * src, unsigned int len) {
 	unsigned int endpos = src->pos + len;
 	int result;
 
-	TRACE(("enter buf_compress"));
+	TRACE(("enter buf_compress"))
 
 	while (1) {
 
@@ -776,6 +606,6 @@ static void buf_compress(buffer * dest, buffer * src, unsigned int len) {
 		buf_resize(dest, dest->size + ZLIB_COMPRESS_INCR);
 
 	}
-	TRACE(("leave buf_compress"));
+	TRACE(("leave buf_compress"))
 }
 #endif
