@@ -22,10 +22,8 @@
  * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
  * SOFTWARE. */
 
-#include <stdlib.h>
-
-#include "options.h"
-#include "util.h"
+#include "includes.h"
+#include "dbutil.h"
 #include "queue.h"
 
 void initqueue(struct Queue* queue) {
@@ -54,7 +52,7 @@ void* dequeue(struct Queue* queue) {
 	} else {
 		queue->head = NULL;
 		queue->tail = NULL;
-		TRACE(("empty queue dequeing"));
+		TRACE(("empty queue dequeing"))
 	}
 
 	m_free(oldhead);
@@ -72,7 +70,7 @@ void enqueue(struct Queue* queue, void* item) {
 
 	struct Link* newlink;
 
-	TRACE(("enter enqueue"));
+	TRACE(("enter enqueue"))
 	newlink = (struct Link*)m_malloc(sizeof(struct Link));
 
 	newlink->item = item;
@@ -87,5 +85,5 @@ void enqueue(struct Queue* queue, void* item) {
 		queue->head = newlink;
 	}
 	queue->count++;
-	TRACE(("leave enqueue"));
+	TRACE(("leave enqueue"))
 }
