@@ -28,18 +28,35 @@
 #include "includes.h"
 #include "signkey.h"
 #include "buffer.h"
+#include "auth.h"
+#include "tcpfwd.h"
 
-struct RunOpts {
+typedef struct runopts {
+
+	int nolocaltcp;
+	int noremotetcp;
+
+} runopts;
+
+extern runopts opts;
+
+int readhostkey(const char * filename, sign_key * hostkey, int *type);
+
+typedef struct svr_runopts {
 
 	char * rsakeyfile;
 	char * dsskeyfile;
 	char * bannerfile;
+
 	int forkbg;
+	int usingsyslog;
 
 	/* ports is an array of the portcount listening ports */
-	uint16_t *ports;
+	char *ports[DROPBEAR_MAX_PORTS];
 	unsigned int portcount;
 
+	int inetdmode;
+
 	/* Flags indicating whether to use ipv4 and ipv6 */
 	/* not used yet
 	int ipv4;
@@ -56,19 +73,41 @@ struct RunOpts {
 	int noauthpass;
 	int norootpass;
 
-	int nolocaltcp;
-	int noremotetcp;
-	
 	sign_key *hostkey;
 	buffer * banner;
 
-	int inetdmode;
+} svr_runopts;
 
-};
+extern svr_runopts svr_opts;
 
-typedef struct RunOpts runopts;
+void svr_getopts(int argc, char ** argv);
+void loadhostkeys();
 
-runopts * getrunopts(int argc, char ** argv);
-void freerunopts(runopts* opts);
+/* Uncompleted XXX matt */
+typedef struct cli_runopts {
+
+	char *progname;
+	char *remotehost;
+	char *remoteport;
+
+	char *username;
+
+	char *cmd;
+	int wantpty;
+#ifdef ENABLE_CLI_PUBKEY_AUTH
+	struct PubkeyList *pubkeys; /* Keys to use for public-key auth */
+#endif
+#ifdef ENABLE_CLI_REMOTETCPFWD
+	struct TCPFwdList * remotefwds;
+#endif
+#ifdef ENABLE_CLI_LOCALTCPFWD
+	struct TCPFwdList * localfwds;
+#endif
+	/* XXX TODO */
+
+} cli_runopts;
+
+extern cli_runopts cli_opts;
+void cli_getopts(int argc, char ** argv);
 
 #endif /* _RUNOPTS_H_ */
